      subroutine xI2qiqi(z,I2qiqi)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqi(-1:4)
      real(dp) :: I2qqVz(-1:4),I2qqSz

      call xI2qqV(z,I2qqVz)
      call xI2qqS(z,I2qqSz)

      I2qiqi(-1:3) = I2qqVz(-1:3)*CF
      I2qiqi(4) = (I2qqVz(4) + I2qqSz)*CF

      return
      end

c---  RB MODIFY: POLARIZED I2QIQI

      subroutine xI2qiqipol(z,I2qiqi)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqi(-1:4)
      real(dp) :: I2qqVz(-1:4),I2qqSpol

      call xI2qqVpol(z,I2qqVz)

c---  divide  by 4 to match normalization of I2 in Eq. 2.28 of 1401.5478     
      I2qiqi(-1:3) = 0.25_dp*I2qqVz(-1:3)
      I2qiqi(4) = 0.25_dp*(I2qqVz(4) + I2qqSpol(z))

      return
      end
      
c---  END RB MODIFY
      
!---
      function I2qiqj(z)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqj
      real(dp) :: I2qqSz

      call xI2qqS(z,I2qqSz)
      I2qiqj = (I2qqSz)*CF

      return
      end

c---  RB MODIFY: ADD POLARIZED I2 FROM EQ. 66 OF 1704.05457

      function I2qqSpol(z)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qqSpol
      real(dp) :: Li31z,Li2z,Lz,L1z,Li3_TR,Li2_TR
      
      Li31z = Li3_TR(1.0_dp-z)
      Li2z = Li2_TR(z)
      Lz = log(z)
      L1z = log(1.0_dp-z)

      I2qqSpol = (1+z)*(-8*Li31z+8*Lz*Li2z-8*L1z*Li2z+10.0_dp*Lz**3
     &     /3.0_dp-4*Lz*L1z**2-8.0_dp*pi**2/3.0_dp*Lz+4.0_dp*pi**2
     &     /3.0_dp*L1z)+(1-z)*(4*Li2z-4*pi**2+(13-17*z)/(1-z)*Lz**2
     &     +10*L1z**2-20*Lz*L1z)+(38-2*z)*Lz-24*(1-z)*L1z+64*(1-z)
      
      I2qqSpol = CF*TR*I2qqSpol

      return
      end

      function I2qiqjpol(z)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqjpol,I2qqSpol

      I2qiqjpol = I2qqSpol(z)

c---  divide by 4 to match normalization of I2 in Eq. 2.28 of 1401.5478
      I2qiqjpol = 0.25_dp*I2qiqjpol
      
      return
      end
      
c---  END RB MODIFY
      
!---
      function I2qiqbi(z)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqbi
c      real(dp) :: I2qqbV,I2qqS
      real(dp) :: I2qqbVz,I2qqSz

      call xI2qqbV(z,I2qqbVz)
      call xI2qqS(z,I2qqSz)

      I2qiqbi = (I2qqbVz + I2qqSz)*CF

      return
      end

!---
      function I2qiqbj(z)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqbj
      real(dp) :: I2qqSz

      call xI2qqS(z,I2qqSz)

      I2qiqbj = (I2qqSz)*CF

      return
      end

c---  RB MODIFY: IQQBVPOL FROM EQ. 67 OF 1704.05457
c---  FP MODIFY: AGREES WITH MATHEMATICA
      function I2qqbVpol(z)
      implicit none
      include 'types.f'
      include 'constants.f'
      include 'nf.f'
      include 'scet_const.f'
      
      real(dp), intent(in) :: z
      real(dp) :: I2qqbVpol
      real(dp) :: Li3a,Li3b,Li3c,Li3d,Li3e
      real(dp) :: Li2a,Li2b,Lz,L1z,ln2
      real(dp) :: Li2_TR,Li3_TR,L1pz
      
      Li3a = Li3_TR((1-z)/2.0_dp)
      Li3b = Li3_TR(1-z)
      Li3c = Li3_TR(z)
      Li3d = Li3_TR((1-z)/(1+z))
      Li3e = Li3_TR((1+z)/2.0_dp)
      Li2a = Li2_TR(1.0_dp/(1+z))
      Li2b = Li2_TR(z)
      Lz = log(z)
      L1z = log(1-z)
      L1pz = log(1+z)
      ln2 = log(2.0_dp)
      
      I2qqbVpol = 16*(1+z**2)/(1-z)*(Li3a-Li3b-0.5_dp*Li3c+Li3d
     &     +Li3e+(Lz-L1z)*Li2a-Lz**3/8.0_dp-0.5_dp*zeta3-L1pz**3/6.0_dp
     &     -ln2**3/3.0_dp+0.5_dp*L1pz**2*Lz+0.5_dp*ln2**2
     &     *log(1-z**2)-pi**2/6.0_dp*L1pz-ln2*L1z*L1pz
     &     +pi**2/12.0_dp*L1z+pi**2/6.0_dp*ln2)-8*(1+z)*Li2a
     &     -8*(1+z)*Li2b-8*z*Lz**2-4*(1+z)*L1pz**2+2*(1+z)*pi**2
     &     +2*(3+19*z)*Lz+16*(1-z)*L1z+30*(1-z)
      
      I2qqbVpol = CF*(0.5_dp*CA-CF)*I2qqbVpol
      
      return
      end
      
      function I2qiqbjpol(z)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqbjpol,I2qqSpol

      I2qiqbjpol = I2qqSpol(z)

c---  divide by 4 to match normalization of I2 in Eq. 2.28 of 1401.5478
      I2qiqbjpol = 0.25_dp*I2qiqbjpol
      
      return
      end

      function I2qiqbipol(z)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qiqbipol,I2qqSpol,I2qqbVpol

      I2qiqbipol = I2qqSpol(z)+I2qqbVpol(z)

c---  divide by 4 to match normalization of I2 in Eq. 2.28 of 1401.5478
      I2qiqbipol = 0.25_dp*I2qiqbipol
      
      return
      end
      
      
c---  END RB MODIFY

!---
      subroutine xI2qqV(z,I2qqV)
      implicit none
      include 'types.f'
      include 'constants.f'
      include 'nf.f'
      include 'scet_const.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qqV(-1:4),Li2_TR,omz,opz
      real(dp) :: T3z,V3z,U3z,splitT3,splitV3,splitU3

      omz=one-z
      opz=one+z
      T3z = splitT3(z)
      V3z = splitV3(z)
      U3z = splitU3(z)


      I2qqV(3) = CF*(1+z**2)

      I2qqV(2) = be0*(1+z**2)*(-1/4.0_dp)

      I2qqV(1) = CF*(1+z**2)*(-5/6.0_dp*pisq)              
     & +CA*(1+z**2)*(2/3.0_dp-pisq/6.0_dp)      
     & +be0*(1+z**2)*5/6.0_dp

      I2qqV(0) = CF*(1+z**2)*4*zeta3                               
     & +CA*(1+z**2)*(-8/9.0_dp+7/2.0_dp*zeta3)  
     & +be0*(1+z**2)*(-7/9.0_dp+pisq/12.0_dp)


      I2qqV(-1) = CF*7/120.0_dp*pisq**2                                                  
     & +CA*(52/27.0_dp-pisq/6.0_dp-pisq**2/36.0_dp)                  
     & +be0*(41/27.0_dp-5/24.0_dp*pisq-5/6.0_dp*zeta3)

      I2qqV(4) = CF*(-2/(1.0_dp-z)*T3z+(1+z**2)/omz*(V3z-2*U3z)               
     & +(1/(1.0_dp-z)+z)*(3*Li2_TR(z)+9/4.0_dp*log(z)**2             
     & +4*log(z)-pisq/2.0_dp )                   
     & -omz*(log(omz)*log(z)-pisq/6.0_dp)                           
     & +(-1/2.0_dp+3/4.0_dp*z)*log(z)**2                                
     & +(-6+11/2.0_dp*z)*log(omz)                                           
     & -(5/2.0_dp+18*z)*log(z)-(13-15*z)/2.0_dp )                   
     & +CA*((1+z**2)/(1.0_dp-z)*(U3z-log(omz)*log(omz/z)*log(z)  
     & -5/4.0_dp*log(z) )                        

     & -opz*Li2_TR(z) -3/2.0_dp*z*log(z)**2                              
     & +(3-5/2.0_dp*z)*log(omz)                                             
     & -(1-11*z)/2.0_dp*log(z)                                                
     & +(7-11*z)/4.0_dp + (1+3*z)/12.0_dp*pisq)                     
     & +be0*((1+z**2)/omz*(Li2_TR(omz)/2.0_dp+log(omz)*log(z)         
     & -5/8.0_dp*log(z)**2-5/4.0_dp*log(z))  
     & +omz/2.0_dp*(log(omz)+1/2.0_dp)+z*log(z)/2.0_dp )
      return
      end

c---  RB MODIFY: I2QQV FOLLOWING EQ. 82 of 1704.05457
c---  FP MODIFY: NUMERICALLY CHECKED TO AGREE WITH UNPOLARIZED
      subroutine xI2qqVpol(z,I2qqV)
      implicit none
      include 'types.f'
      include 'constants.f'
      include 'nf.f'
      include 'scet_const.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qqV(-1:4)
      real(dp) :: Li2_TR,Li21z,Lz,L1z,Li31z,Li3z,Li3_TR
      real(dp) :: betaterm,CAterm,CFterm
      
      Li21z = Li2_TR(1-z)
      Li31z = Li3_TR(1-z)
      Li3z = Li3_TR(z)
      Lz = log(z)
      L1z = log(1-z)
      
!# delta function term
      I2qqV(-1) = CF*(CF*7.0_dp*pisq**2/30.0_dp+CA*(208.0_dp/27.0_dp
     &     -2.0_dp*pisq/3.0_dp-pisq**2/9.0_dp)+be0*(164.0_dp/27.0_dp
     &     -5.0_dp*pisq/6.0_dp-10.0_dp*zeta3/3.0_dp))

!# plus distribution terms
      I2qqV(3) = 8*CF**2
      I2qqV(2) = -2*CF*be0
      I2qqV(1) = CF**2*(-20.0_dp*pisq/3.0_dp)
     &     -CF*CA*(4.0_dp*pisq/3.0_dp-16.0_dp/3.0_dp)
     &     +CF*be0*20.0_dp/3.0_dp
      I2qqV(0) = 32*CF**2*zeta3+CF*CA*(28*zeta3-64.0_dp/9.0_dp)
     &     +CF*be0*(2.0_dp*pisq/3.0_dp-56.0_dp/9.0_dp)

!# regular term

      betaterm = (1+z**2)/(1-z)*(2*Li21z-2.5_dp*Lz**2+4*Lz*L1z
     &     -pisq/3.0_dp*(1-z**2)/(1+z**2)+L1z**2*(1-z**2)/(1+z**2))
     &     -4.0_dp/3.0_dp*(1+4*z)*L1z-(5-2*z+7*z**2)/(1-z)*Lz
     &     +(37+19*z)/9.0_dp

      CAterm = (1+z**2)/(1-z)*(-16*Li31z+4*Li3z+4*Li21z*L1z+4*Lz**2*L1z
     &     +2.0_dp/3.0_dp*pisq*L1z*(1-z**2)/(1+z**2)+8*Li21z*Lz-Lz**3
     &     -2.0_dp*(9-5*z**2)/(1+z**2)*zeta3)+4*(1+z)*Lz*L1z+4*(1+z)
     &     *Li21z-(1-z)*pisq/3.0_dp-6*z*Lz**2+(28-38*z)*L1z/3.0_dp
     &     +(3+27*z-10.0_dp/(1-z))*Lz+(95-67*z)/9.0_dp

      CFterm = 1.0_dp/(1-z)*(8*(1+2*z**2)*Li31z-28*(1+z**2)*Li3z
     &     -8*z**2*L1z*Li21z+2*(z**2-3)*Lz**2*L1z+10.0_dp/3.0_dp*
     &     pisq*(1-z**2)*L1z+(22+26*z**2)*pisq/3.0_dp*Lz-8*(2*z**2+3)
     &     *Li21z*Lz+4*(3+11*z**2)*zeta3-4*(3+4*z**2)*Lz*L1z**2
     &     -4*(1-z**2)*L1z**3+5.0_dp/3.0_dp*(1-z**2)*Lz**3)
     &     +1.0_dp/(1-z)*(-4*(4+z-2*z**2)*Lz*L1z-12*(1+z-z**2)
     &     *Li21z+2.0_dp/3.0_dp*(1-z)**2*pisq+(7+14*z-12*z**2)*Lz**2)
     &     -2*(12-11*z)*L1z+2*(-5-28*z+8.0_dp/(1-z))*Lz-26+30*z

      I2qqV(4) = CF**2*CFterm+CF*CA*CAterm+CF*be0*betaterm
            
      return
      end subroutine

      
c---  END RB MODIFY
      

!---
      subroutine xI2qqbV(z,I2qqbV)
      implicit none
      include 'types.f'
      include 'constants.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qqbV,omz,opz
      real(dp) :: S3z,Li2_TR,splitS3

      S3z = splitS3(z)
      omz=one-z
      opz=one+z
      I2qqbV = (2*CF-CA)*((1+z**2)/opz*S3z
     &   -opz/2.0_dp*(Li2_TR(z**2)+2*log(z)*log(opz)-pisq/6.0_dp)               
     &   -z*log(z)**2+2*omz*log(omz)          
     &   +(3+19*z)/4.0_dp*log(z)+15/4.0_dp*omz)
      return
      end

!--- 
      subroutine xI2qqS(z,I2qqS)
      implicit none
      include 'types.f'
      include 'constants.f'
c      include 'nf.f'
c      include 'scet_const.f'

      real(dp), intent(in) :: z
      real(dp) :: I2qqS,omz,opz
      real(dp) :: T3z,Li2_TR,splitT3

      omz=one-z
      opz=one+z
      T3z = splitT3(z)

      I2qqS = TR*(-2*opz*T3z                                           
     & -(3+4/3.0_dp/z+5*z+8/3.0_dp*z**2)            
     & *(Li2_TR(z)-pisq/6.0_dp)                           
     & +(1+4/3.0_dp/z-z-4/3.0_dp*z**2)              
     & *(log(omz)**2/2.0_dp                              
     & -log(omz)*log(z) - pisq/6.0_dp)         
     & -(13/4.0_dp*opz+10/3.0_dp*z**2)*log(z)**2              
     & +(26/9.0_dp/z-11/3.0_dp+17/3.0_dp*z-44/9.0_dp*z**2)  
     & *log(omz)                                             
     & +(23/3.0_dp-5/3.0_dp*z+76/9.0_dp*z**2)*log(z)            
     & +104/27.0_dp/z-41/18.0_dp+17/18.0_dp*z-68/27.0_dp*z**2)
      return
      end


!---

      function I2qig(z)
      implicit none
      include 'types.f'
      include 'constants.f'
      include 'nf.f'
      include 'scet_const.f'
  
      real(dp), intent(in) :: z
      real(dp) :: I2qig,omz,opz
      real(dp) :: T3z,V3z,U3z,S3z,S2z
      real(dp) :: Li2_TR,splitS2,splitS3,splitT3,splitU3,splitV3
      real(dp) :: pqgz,pqgmz
  
      omz=one-z
      opz=one+z
      T3z = splitT3(z)
      V3z = splitV3(z)
      U3z = splitU3(z)
      S3z = splitS3(z)
      S2z = splitS2(z)
      pqgz = omz**2+z**2
      pqgmz = opz**2+z**2
  
      I2qig=CF*(-2*omz**2*T3z                                
     &           +pqgz*(V3z+5/6.0_dp*log(omz)**3                
     &                  -log(omz/z)*log(omz)*log(z)           
     &                  -5/6.0_dp*pisq*log(omz)                 
     &                  -pisq/3.0_dp*log(z)+11*zeta3)           
     &           -(7/4.0_dp-7*z+6*z**2)*log(omz/z)**2         
     &           +(7+4*z)/8.0_dp*log(z)**2                      
     &           +Li2_TR(z)/2.0_dp                                 
     &           +(13/2.0_dp-23*z+39/2.0_dp*z**2)*log(omz/z)  
     &           +(3/2.0_dp+9/4.0_dp*z)*log(z)                  
     &           -(34-145*z+121*z**2)/4.0_dp                    
     &           +(3-12*z+10*z**2)*pisq/6.0_dp)                 
     &      +CA*(-2*(1+4*z)*T3z-pqgz*(U3z-log(omz)**3/6.0_dp    
     &           +pisq/6.0_dp*log(omz)-pisq/3.0_dp*log(z)       
     &           +7/2.0_dp*zeta3)+pqgmz*S3z-z*opz*S2z         
     &           -2*z*omz*log(omz)*log(z)                     
     &           +(2/3.0_dp/z+1/2.0_dp+3*z-25*z**2/6.0_dp)      
     &            *log(omz/z)**2                              
     &           -(15/4.0_dp+2/3.0_dp/z+2*z+47/3.0_dp*z**2)     
     &            *log(z)**2                                    
     &           -(4/3.0_dp/z+3+8*z+44/3.0_dp*z**2)*Li2_TR(z)      
     &           +(26/9.0_dp/z+4+31/2.0_dp*z+50/9.0_dp*z**2)    
     &            *log(omz)                                     
     &           -(49/6.0_dp+4/3.0_dp*z+323/18.0_dp*z**2)       
     &            *log(omz/z)                                 
     &           + 104/27.0_dp/z - 19/36.0_dp + 40*z/9.0_dp     
     &           - 947/108.0_dp*z**2 + (2+z+24*z**2)/6.0_dp*pisq)
  
       I2qig = I2qig*TR
    
       return  
       end

c---  RB MODIFY: ADD POLARIZED I2QG FROM EQ. 79 OF 704.05457
c---  FP MODIFY: AGREES WITH MATHEMATICA
      function I2qigpol(z)
      implicit none
      include 'types.f'
      include 'constants.f'
      include 'nf.f'
      include 'scet_const.f'
  
      real(dp), intent(in) :: z
      real(dp) :: I2qigpol,CFterm,CAterm
      real(dp) :: Li3z,Li31z,Li2z,Lz,L1z,L1pz
      real(dp) :: Li2_TR,Li3_TR
      real(dp) :: Li3a,Li3b,Li3c,Li2a
      
      Li3z = Li3_TR(z)
      Li31z = Li3_TR(1-z)
      Li2z = Li2_TR(z)
      Lz = log(z)
      L1z = log(1-z)
      L1pz = log(1+z)
      Li3a = Li3_TR((1-z)/2.0_dp)
      Li3b = Li3_TR((1-z)/(1+z))
      Li3c = Li3_TR((1+z)/2.0_dp)
      Li2a = Li2_TR(1.0_dp/(1+z))
      
      CFterm = 2*(1-2*z)*(10*Li3z+12*Li31z+4*L1z*Li2z+pi**2*L1z
     &     -3*L1z*Lz**2-4*Li2z*Lz-7.0_dp/3.0_dp*pi**2*Lz-32*zeta3
     &     -5.0_dp/3.0_dp*L1z**3+Lz**3/6.0_dp+8*Lz*L1z**2)
     &     -6*(7-8*z)*Lz*L1z
     &     -2*(3-4*z)*Li2z-2.0_dp/3.0_dp*(7-8*z)*pi**2+(21-24*z)*L1z**2
     &     +(37.0_dp/2.0_dp-12*z)*Lz**2-12*(5-6*z)*L1z+(70-63*z)*Lz
     &     +99-109*z

      CAterm = 2*(1+2*z)*(4*Li3a+4*Li3b+4*Li3c+4*(Lz-L1z)*Li2a+2*(Lz
     &     -L1pz)*L1pz**2+4.0_dp/3.0_dp*(log((1+z)/2.0_dp))**3+
     &     2*log(2.0_dp)*(L1pz-L1z)*log((1+z)**2/2.0_dp)-2.0_dp
     &     /3.0_dp*pi**2*log((1+z)/2.0_dp))
     &     -2*(1-2*z)*(4*Lz**2*L1z+L1z**3/3.0_dp)-40*Li31z-16*z*Li3z
     &     -4*(5+2*z)*Li2z*L1z+2.0_dp/3.0_dp*pi**2*(7+2*z)*L1z
     &     +2.0_dp/3.0_dp*(7+10*z)*Lz**3+8*(1+4*z)*Li2z*Lz
     &     -16.0_dp/3.0_dp*(1+z)*Lz*pi**2+2*(3-14*z)*zeta3
     &     -8*(1+z)*Lz*L1z**2-4*(1+4*z)*Li2z+8*(1+z)*Li2a
     &     -48*(1-z)*Lz*L1z-2.0_dp/3.0_dp*(11-14*z)*pi**2+20*(1-z)
     &     *L1z**2+(29-44*z)*Lz**2+4*(1+z)*L1pz**2-2*(12-13*z)*L1z
     &     +80*Lz+2*(57-59*z)


      I2qigpol = TR*(CF*CFterm+CA*CAterm)

c---  divide by 4 to match normalization of I2 in Eq. 2.28 of 1401.5478
      I2qigpol = 0.25_dp*I2qigpol
      
      return
      end function

c---  END RB MODIFY

      function I2gqi(z)
      implicit none
      include 'types.f'
      include 'constants.f'
      include 'nf.f'
      include 'scet_const.f'
 
      real(dp), intent(in) :: z
      real(dp) :: I2gqi,omz,opz
      real(dp) :: T3z,V3z,U3z,S3z,S2z
      real(dp) :: Li2_TR,splitS2,splitS3,splitT3,splitU3,splitV3
      real(dp) :: pgqz,pgqmz
 
      omz=one-z
      opz=one+z
      T3z = splitT3(z)
      V3z = splitV3(z)
      U3z = splitU3(z)
      S3z = splitS3(z)
      S2z = splitS2(z)
      pgqz = (1+omz**2)/z
      pgqmz = (1+opz**2)/(-z)
 
      I2gqi = CA*(pgqz*(V3z-U3z-T3z                                                   
     &            +5/6.0_dp*log(omz)**3-2/3.0_dp*pisq*log(omz)                  
     &            -pisq/3.0_dp*log(z)+45/6.0_dp*zeta3)                          
     &      +2*(4+z)*T3z+pgqmz*S3z-z/2.0_dp*(S2z-pisq/2.0_dp)                   
     &      +(-31/6.0_dp/z+4+2*z+2/3.0_dp*z**2)
     &       *(log(omz/z)**2-pisq/3.0_dp)   
     &      +(11/3.0_dp/z+15+11/4.0_dp*z+8/3.0_dp*z**2)*log(z)**2               
     &      +(22/3.0_dp/z+14+5*z+8/3.0_dp*z**2)*(Li2_TR(z)-pisq/6.0_dp)            
     &      +(-181/18.0_dp/z+37/3.0_dp-6*z+44/9.0_dp*z**2)*log(omz)             
     &      -(43/6.0_dp/z+51/2.0_dp+13/6.0_dp*z+88/9.0_dp*z**2)*log(z)          
     &      -2351/108.0_dp/z+101/6.0_dp-83/36.0_dp*z+152/27.0_dp*z**2 )         
     &  +CF*(pgqz*(log(omz)**3/6.0_dp-log(omz/z)*log(omz)*log(z)              
     &             -pisq/3.0_dp*log(omz/z)) - (2-z)*T3z                       
     &      +(-9/2.0_dp/z+11/2.0_dp-3*z)*(log(omz/z)**2-pisq/3.0_dp)          
     &      +(-3/z+3-5/2.0_dp*z)*(log(omz)*log(z)+pisq/6.0_dp)                  
     &      +(9/2.0_dp/z-7+9/8.0_dp*z)*log(z)**2                                
     &      -(6+5*z)/2.0_dp*(Li2_TR(z)-pisq/6.0_dp)                                
     &      +(21/2.0_dp/z-75/6.0_dp)*log(omz)                                   
     &      +(-15/6.0_dp/z+9/4.0_dp+15/4.0_dp*z)*log(z)                         
     &      -43/4.0_dp/z+16-3*z )                                               
     &  +be0*(pgqz*(log(omz)**2/4.0_dp-log(z)**2/2.0_dp                         
     &        +5/6.0_dp*log(omz)-5/3.0_dp*log(z)-14/9.0_dp)                     
     &       +z/2.0_dp*(log(omz)+5/3.0_dp) )
 
      I2gqi = I2gqi*CF
  
 !     write(*,*)"I2gqi:",I2gqi,"z:",z
 !     pause
 
      return
 
      end function I2gqi
 
