      subroutine nplotter(p,wt,wt2,nd)
      implicit none
      include 'types.f'
c--- Variable passed in to this routine:
c
c---      p:  4-momenta of leptons and jets in the format p(i,4)
c---          with the particles numbered according to the input file
c---          and components labelled by (px,py,pz,E)
c
c---     wt:  weight of this event
c
c---    wt2:  weight^2 of this event
c
c---     nd:  an integer:: specifying the dipole number of this contribution
c---          (if applicable), otherwise equal to zero
      
      include 'constants.f'
      include 'nf.f'
      include 'mxpart.f'
      include 'cplx.h'
      include 'kprocess.f'
      include 'nplot.f'
      include 'nproc.f'
      include 'first.f'
      include 'taucut.f'

c--- APPLgrid - use of grids
c      include 'ptilde.f'
c      include 'APPLinclude.f'
c--- APPLgrid - end


      real(dp):: p(mxpart,4),wt,wt2
      integer:: switch,nd
      integer, save :: plotindex
!$omp threadprivate(plotindex)
      
c--- This routine simply picks out a process-specific plotting routine
c---  (if available) and falls back to the generic routine otherwise.
c---  So far available: W_only, Z_only, Wbbbar, Wbbmas, WpWp2j, WpWp3j
c---  For the convenience of the user who wants to bail out and do their
c---  own plotting we provide the dummy routine userplotter

c---  the index of the plot - stored in the nplot.f common and used for                                                      
c---  keeping track of the plot index across different files (GPS)                                                           
      nextnplot = 1

c---  first allow for user plots
      call userplotter(p,wt,wt2,nd)

c--- switch:  an integer:: equal to either 0 or 1, depending on the type of event
c---                0  --> lowest order, virtual or real radiation
c---                1  --> counterterm for real radiation
      if (nd>0) then 
         switch=1
      else
         switch=0
      endif
         
c--- work out which plotting routine to use when first called
c-----> saves string comparison in general and important for combining SCET
      if (first) then
        first=.false.
        if     (kcase==kW_only) then
c          plotindex=1
          plotindex=1000 ! revert to default plotting routine
        elseif (kcase==kZ_only) then
          plotindex=2
        elseif (kcase==kW_cjet) then
          plotindex=3
        elseif (kcase==kWbbbar) then
          plotindex=4
        elseif ((kcase==kWbbmas) .or. (kcase==kW_bjet)) then
          plotindex=5
        elseif ((kcase==kWpWp2j) .or. (kcase==kWpWp3j))then
          plotindex=6
        elseif ((kcase==kW_1jet) .or. (kcase==kW_2jet) 
     &     .or. (kcase==kW_3jet)) then
          plotindex=7
        elseif ((kcase==kHWW_4l) .or. (kcase==kHWW_tb) 
     &     .or. (kcase==kHWW2lq) .or. (kcase==kHWWint) 
     &     .or. (kcase==kHWWHpi) .or. (kcase==kggWW4l)
     &     .or. (kcase==kWWqqbr) .or. (kcase==kggWWbx)) then
          plotindex=8
        elseif ((kcase==kWW_jet) .or. (kcase==kWW2jet)) then
          plotindex=9
c--- photon processes also need to know the dipole number
        elseif ((kcase==kWgamma) .or. (kcase==kZgamma)
     &     .or. (kcase==kWgajet) .or. (kcase==kZgajet)) then
          plotindex=10
        elseif ((kcase==kgamgam) .or. (kcase==kgg2gam)) then
          plotindex=11
        elseif (kcase==kgmgmjt) then
          if (usescet) then
            plotindex=11 ! same routine below and above cut for SCET
          else
            plotindex=12
          endif
        elseif (kcase==kdirgam) then 
          plotindex=13
        elseif (kcase==ktrigam) then 
          plotindex=14
        elseif (kcase==kW_2gam)  then
          plotindex=15
        elseif (kcase==kZ_2gam)  then
          plotindex=16
        elseif (kcase==kZgajet)  then
          plotindex=17
        elseif ((kcase==ktt_bbl) .or. (kcase==ktt_ldk)
     &     .or. (kcase==ktt_bbh) .or. (kcase==ktt_bbu)
     &     .or. (kcase==ktt_hdk) .or. (kcase==ktthWdk)
     &     .or. (kcase==ktt_udk)) then
          plotindex=18
        elseif ((kcase==k4ftwdk) .or. (kcase==kdk_4ft)) then
          plotindex=19
        elseif ((kcase==kt_bbar) .or. (kcase==ktdecay)) then
          plotindex=20
        elseif (kcase==kqq_ttw) then
          plotindex=21
        elseif ((kcase==kH_tjet) .or. (kcase==kZ_tjet)) then
          plotindex=22
        elseif ((kcase==kH_tdkj) .or. (kcase==kZ_tdkj)) then
          plotindex=23
        elseif (kcase==kqqtthz) then
          plotindex=24
        elseif ((kcase==kdm_jet).or.(kcase==kdm2jet)) then 
          plotindex=25
        elseif ((kcase==kdm_gam).or.(kcase==kdm_gaj)) then 
          plotindex=26
        elseif ((kcase==kqqZZqq).or.(kcase==kqqWWqq) 
     &     .or. (kcase==kqqVVqq).or.(kcase==kqqWWss)
     &     .or. (kcase==kqqWZqq).or.(kcase==kWpmZjj)
     &     .or.(kcase==kqq_ttg)) then 
          plotindex=27
        elseif ((kcase==kHZZ_4l)
     &    .or.  (kcase==kHZZ_tb)
     &    .or.  (kcase==kHZZint)
     &    .or.  (kcase==kHZZHpi)
     &    .or.  (kcase==kggZZ4l) 
     &    .or.  (kcase==kggZZbx) 
     &    .or.  (kcase==kHZZqgI) 
     &    .or.  (kcase==kZZlept)
     &    .or.  (kcase==kggVV4l) 
     &    .or.  (kcase==kggVVbx) 
     &    .or.  (kcase==kHVV_tb)) then 
           plotindex=28
        elseif((kcase.eq.kWHbbar).or.(kcase.eq.kZHbbar)
     &     .or.(kcase.eq.kWH1jet).or.(kcase.eq.kZH1jet)
     &     .or.(kcase.eq.kWHbbdk).or.(kcase.eq.kZHbbdk)
     &          ) then
           plotindex=29
        elseif (kcase==kZbbbar) then
           plotindex=30
        else
          plotindex=1000
        endif
      endif

        call nplotter_W_only(p,wt,wt2,switch)
      
c--- APPLgrid - filling applgrid
c      if (creategrid) call fill_grid(p)
c--- APPLgrid - end

      return
      end
      
